/**
 * Copyright (c) 2014 - 2019, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 * @defgroup nrf_dev_timer_example_main main.c
 * @{
 * @ingroup nrf_dev_timer_example
 * @brief Timer Example Application main file.
 *
 * This file contains the source code for a sample application using Timer0.
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include "nrf.h"
#include "nrf_drv_timer.h"
#include "bsp.h"
#include "app_error.h"
#include "nrf_delay.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#define TIMER_COMPARE_FREQUENCY_US              1000                   // Number of us needed in this application 
#define TIMER_BASE_FREQUENCY                    NRF_TIMER_FREQ_16MHz   // Timer operating frequency


const nrfx_timer_t TIMER = NRF_DRV_TIMER_INSTANCE(1);
static uint32_t timer_ticks;
uint32_t i = 0;
/**
 * @brief Handler for timer events.
 */
void timer_led_event_handler(nrf_timer_event_t event_type, void* p_context)
{
    static uint32_t led = 0;
    static uint32_t old_timer_value;
    static uint32_t capture_values[100] = {0};
  
    
    switch (event_type)
    {
        case NRF_TIMER_EVENT_COMPARE1:
            // Update timer to reach next N ms
            NRF_TIMER1 -> CC[1] += timer_ticks;
            
            // Capture CHANNEL0
            NRF_TIMER1 -> TASKS_CAPTURE[0] = 1;
            
            // Calculate delta time from last capture
            uint32_t captured_timer_value = NRF_TIMER1 -> CC[0];
            uint32_t delta_ticks = captured_timer_value - old_timer_value;
            if(delta_ticks != timer_ticks)
            {
              NRF_LOG_WARNING("Timer delta measured %d, target %d", delta_ticks, timer_ticks);
              //NRF_TIMER1 -> CC[1] = captured_timer_value + timer_ticks;
              //NRF_LOG_WARNING("UPDATE TIMER1 compare value, due not meeting timing requirement");
              NRF_LOG_FLUSH();
             }
            
            i++;
            // Toggle some LEDs
            if((i % 100) == 0)
            {
              uint32_t led_to_invert = ((led++) % LEDS_NUMBER);
              bsp_board_led_invert(led_to_invert);
              
            }  
            
//            // Calculate average capture ticks every 100th time
//            capture_values[i] = delta_ticks;
//            if(++i == 100)
//            {
//                uint32_t average_delta = 0;
//                for(uint8_t j=0; j<100; j++)
//                {
//                  average_delta += capture_values[j];
//                }
//                average_delta = average_delta/100;
//
//                NRF_LOG_INFO("Average delta ticks %d, target number of ticks %d", average_delta, timer_ticks);
//                NRF_LOG_FLUSH();
//                i = 0;
//            }
            
            
            old_timer_value = captured_timer_value;
            break;

        default:
            //Do nothing.
            break;
    }
}

/**@brief Function for initializing logging. */
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}

/**
 * @brief Function for main application entry.
 */
int main(void)
{

    log_init();
    NRF_LOG_INFO("Timer example");
    
    uint32_t err_code = NRF_SUCCESS;
    
    //Configure all leds on board.
    bsp_board_init(BSP_INIT_LEDS);

    //Configure TIMER for generating simple light effect - leds on board will invert his state one after the other.
    nrfx_timer_config_t timer_cfg = NRFX_TIMER_DEFAULT_CONFIG;
    timer_cfg.frequency = TIMER_BASE_FREQUENCY;
    
    // Initialize configured TIMER
    err_code = nrfx_timer_init(&TIMER, &timer_cfg, timer_led_event_handler);
    APP_ERROR_CHECK(err_code);

    // Calculate number of TIMER ticks needed for N us frequency
    timer_ticks = nrfx_timer_us_to_ticks(&TIMER, TIMER_COMPARE_FREQUENCY_US);
    NRF_LOG_INFO("Timer ticks = %d", timer_ticks);
    NRF_LOG_FLUSH();

    // Setup compare event on CHANNEL1
    //nrfx_timer_extended_compare(&TIMER, NRF_TIMER_CC_CHANNEL1, time_ticks, 0, true);
    nrfx_timer_compare(&TIMER, NRF_TIMER_CC_CHANNEL1, timer_ticks, true);

    // Start TIMER
    nrfx_timer_enable(&TIMER);

    
    while (1)
    {
        __WFI();
        // Add timing violation to delay timer callback function
        if(i == 10000)
        {
            NRF_TIMER1 -> CC[1] += 50;
            NRF_LOG_INFO("Add some delay");
            NRF_LOG_FLUSH();
            i = 0;
        }
    }
}

/** @} */
